#!/usr/bin/env python

#======================================================================#
#
#======== Pressure Sensor Demonstration Script 05/04/2014 ==========#
#
#  Demo script for Pressure Sensor
#
#
# www.circuitsurgery.co.uk
#
#======================================================================#

debug = False

print ""
print "================ Barometric Pressure Sensor Demonstration ======================"
print "                ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
print ""
print "www.circuitsurgery.co.uk"
print ""

import sys
import smbus
import time
import RPi.GPIO as gpio
import spidev

gpio.setmode(gpio.BCM)
gpio.setwarnings(False)

bus = smbus.SMBus(1)
spi = spidev.SpiDev()

#======================================================================#
#	Variables
#----------------------------------------------------------------------#

accel_interrupt = 4
halln = 15
halls = 14
barrier_send = 23
barrier_receive = 24
ip = [halls,halln,barrier_receive,accel_interrupt]		# Input ports
op = [barrier_send]			# Output ports
ball = 0b00011000	# Initial position of "ball" for accelerometer demo
slp = 1.1	# Create floating point variable

results = bytearray()

# Pressure variables (as per data sheet)
# Padc		Pressure ADC value
# Padc_LSB	Pressure ADC value LSB (2 bits)
# Padc_MSB	Pressure ADC value MSB (8 bits)
# Tadc		Temperature ADC value
# Tadc_LSB	Temperature ADC value LSB (2 bits)
# Tadc_MSB	Temperature ADC value MSB (8 bits)

# a0		a0 coefficient
# a0_MSB	a0 coefficient MSBip = [halls,halln,barrier_receive,accel_interrupt]		# Input ports
op = [barrier_send]			# Output ports
# a0_LSB	a0 coefficient LSB

# b1		b1 coefficient
# b1_MSB	b1 coefficient MSB
# b1_LSB	b1 coefficient LSB

# b2.0		b2 coefficient
# b2_MSB	b2 coefficient MSB
# b2_LSB	b2 coefficient LSB

# c12.0		c12 coefficient
# c12_MSB	c12 coefficient MSB
# c12_LSB	c12 coefficient LSB

# Pcomp		Compensated pressure measurement
# Pressure	Pressure in kPa


#======================================================================#
#	Functions
#----------------------------------------------------------------------#


#======================================================================#
#	Addresses
#----------------------------------------------------------------------#
addr_temperature = 0x48
addr_accelerometer = 0x4C
addr_light = 0x29
addr_pressure = 0x60
addr_expander = 0x20
#======================================================================#

#======================================================================#
#	REGISTERS
#----------------------------------------------------------------------#

press_reg_pressure_msb = 0x00
press_reg_pressure_lsb = 0x01
press_reg_temp_msb = 0x02
press_reg_temp_lsb = 0x03
press_reg_A0_coeff_msb = 0x04
press_reg_A0_coeff_lsb = 0x05
press_reg_B1_coeff_msb = 0x06
press_reg_B1_coeff_lsb = 0x07
press_reg_B2_coeff_msb = 0x08
press_reg_B2_coeff_lsb = 0x09
press_reg_C12_coeff_msb = 0x0A
press_reg_C12_coeff_lsb = 0x0B
press_reg_startconversion = 0x12

#======================================================================#
#	BITS in REGISTERS
#----------------------------------------------------------------------#

#======================================================================#
# set GPIO ports
#----------------------------------------------------------------------#
for n in range(4):
	gpio.setup(ip[n],gpio.IN)

gpio.setup(op[0],gpio.OUT)
	
#======================================================================#

#======================================================================#
#======================================================================#
# ---------- SET EXPANSION PORTS FOR OUTPUT ----------
#----------------------------------------------------------------------#

spi.open(0,0)

# Set port expander ports to o/p
resp = spi.xfer2([0x40,0x00,0x00])	# device address, register 00, set to 00
# Set all bits to "0"
resp = spi.xfer2([0x40,0x09,0x00])	# device address, register 09, set to 00

#============================================================================

# ** Set up Pressure Sensor **
# ********* MPL115A2 *********

# Get and process coefficients (One time event)

results = bus.read_i2c_block_data(addr_pressure, 0x00)

# a0 -----------------------------------------------
a0_MSB = results[4]			# a0 coefficient MSB
a0_LSB = results[5]			# a0 coefficient LSB

if debug:
	a0_MSB = 0x3E
	a0_LSB = 0xCE

a0 = a0_MSB<<8 | a0_LSB		# 16-bit number (1 sign bit, 12 integer bits, 3 fractional bits)
if debug:
	print "raw a0 = ", hex(a0)
a0 = float(a0)
a0 /= 2**3

# b1 -----------------------------------------------
b1_MSB = results[6]			# b1 coefficient MSB
b1_LSB = results[7]			# b1 coefficient LSB

if debug:
	b1_MSB = 0xB3
	b1_LSB = 0xF9

b1 = b1_MSB<<8 | b1_LSB		# 16-bit number (1 sign bit, 2 integer bits, 13 fractional bits)
if debug:
	print "raw b1 = ", hex(b1)
if b1&0x8000 <> 0:	# Test for negative numberresults = bus.read_i2c_block_data(addr_pressure, 0x00)
	b1 ^= 0xFFFF	# Negative number, so flip bits...
	b1 += 1			#... and add 1
	b1 = float(b1)
	b1 /= 2.0**13
	b1 *= -1		# Make it negative
else:
	b1 = float(b1)
	b1 /= 2.0**13

# b2 -----------------------------------------------
b2_MSB = results[8]			# b2 coefficient MSB
b2_LSB = results[9]			# b2 coefficient LSB

if debug:
	b2_MSB = 0xC5
	b2_LSB = 0x17

b2 = b2_MSB<<8 | b2_LSB		# 16-bit number (1 sign bit, 1 integer bits, 14 fractional bits)
if debug:
	print "raw b2 = ", hex(b2)
if b2&0x8000 <> 0:	# Test for negative number
	b2 ^= 0xFFFF	# Negative number, so flip bits...
	b2 += 1			#... and add 1
	b2 = float(b2)
	b2 /= 2**14		# divide by 2^(fractional bits)
	b2 *= -1		# Make it negative
else:
	b2 = float(b2)
	b2 /= 2**14		# divide by 2^(fractional bits)

# c12 ----------------------------------------------
c12_MSB = results[10]	# c12 coefficient MSB
c12_LSB = results[11]	# c12 coefficient LSB

if debug:
	c12_MSB = 0x33
	c12_LSB = 0xC8

c12 = c12_MSB<<8 | c12_LSB	# 14-bit number (1 sign bit, 0 integer bits, 13 fractional bits)
c12 = c12>>2		# 14-bit number, so get rid of last 2 unused bits
if debug:
	print "raw c12 = ", hex(c12)
if c12&0x2000 <> 0:	# Test for negative number
	c12 ^= 0x3FFF	# Negative number, so flip bits...
	c12 += 1		#... and add 1
	c12 = float(c12)
	c12 /= 2**22	# divide by 2^(fractional bits)
	c12 *= -1		# Make it negative
else:
	c12 = float(c12)
	c12 /= 2**22	# divide by 2^(fractional bits)

if debug:
	print "a0 = ", a0, "(Expected - 2009.75)"
	print "b1 = ", b1, "(Expected - -2.37585)"
	print "b2 = ", b2, "(Expected - -0.92047)"
	print "c12 = ", c12, "(Expected - -0.000790)"


# Make sure LED's are off!
spi.open(0,0)
spi.xfer2([0x40,0x00,0x00])	#Make all expansion ports o/p
spi.xfer2([0x40,0x09,0x00])	#Make all expansion ports "0"results = bus.read_i2c_block_data(addr_pressure, 0x00)
spi.close()

while True:
# Start conversion
	bus.write_byte_data(addr_pressure, 0x12, 0)
	time.sleep(0.003)	# Wait for conversion to complete
	results = bus.read_i2c_block_data(addr_pressure, 0x00)
	Padc_MSB = results[0]
	Padc_LSB = results[1]

	if debug:
		Padc_MSB = 0x66
		Padc_LSB = 0x80

	Padc = Padc_MSB<<8 | Padc_LSB
	Padc = Padc>>6	# 10-bit number

	Tadc_MSB = results[2]
	Tadc_LSB = results[3]

	if debug:
		Tadc_MSB = 0x7E
		Tadc_LSB = 0xC0

	Tadc = Tadc_MSB<<8 | Tadc_LSB
	Tadc = Tadc>>6	# 10-bit number

	# The raw results are in, now to do the compensation calculations

	# Equation 1:- Pcomp = a0+(b1+c12*Tadc)*Padc+b2*Tadc
	Pcomp = a0+(b1+c12*Tadc)*Padc+b2*Tadc
	# Equation 2:- Pressure (kPa) = Pcomp*((115-50)/1023)+50
	Pressure = Pcomp*((115.0-50.0)/1023.0)+50.0
	print ""
	print "The current atmospheric pressure is ", Pressure,"kPa"
	print ""
	time.sleep(1)

